@@ -2,16 +2,16 @@ import logging
 
 logging.info(
     "Hello world!",
-    extra=dict(
-        name="foobar",
-    ),
+    extra={
+        "name": "foobar",
+    },
 )
 
 from logging import info
 
 info(
     "Hello world!",
-    extra=dict(
-        name="foobar",
-    ),
+    extra={
+        "name": "foobar",
+    },
 )
