@@ -1,5 +1,5 @@
 use ruff_formatter::{
-    write, FormatContext, FormatOwnedWithRule, FormatRefWithRule, FormatRuleWithOptions,
+    FormatContext, FormatOwnedWithRule, FormatRefWithRule, FormatRuleWithOptions, write,
 };
 use ruff_python_ast::helpers::is_compound_statement;
 use ruff_python_ast::{self as ast, Expr, PySourceType, Stmt, Suite};
@@ -8,11 +8,14 @@ use ruff_python_trivia::{lines_after, lines_after_ignoring_end_of_line_trivia, l
 use ruff_text_size::{Ranged, TextRange};
 
 use crate::comments::{
-    leading_comments, trailing_comments, Comments, LeadingDanglingTrailingComments,
+    Comments, LeadingDanglingTrailingComments, leading_comments, trailing_comments,
 };
 use crate::context::{NodeLevel, TopLevelStatementPosition, WithIndentLevel, WithNodeLevel};
-use crate::expression::expr_string_literal::ExprStringLiteralKind;
+use crate::other::string_literal::StringLiteralKind;
 use crate::prelude::*;
+use crate::preview::{
+    is_allow_newline_after_block_open_enabled, is_blank_line_before_decorated_class_in_stub_enabled,
+};
 use crate::statement::stmt_expr::FormatStmtExpr;
 use crate::verbatim::{
     suppressed_node, write_suppressed_statements_starting_with_leading_comment,
@@ -20,7 +23,7 @@ use crate::verbatim::{
 };
 
 /// Level at which the [`Suite`] appears in the source code.
-#[derive(Copy, Clone, Debug, Default, PartialEq, Eq)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq)]
 pub enum SuiteKind {
     /// Statements at the module level / top level
     TopLevel,
@@ -32,23 +35,62 @@ pub enum SuiteKind {
     Class,
 
     /// Statements in any other body (e.g., `if` or `while`).
-    #[default]
-    Other,
+    Other {
+        /// Whether this suite is the last suite in the current statement.
+        ///
+        /// Below, `last_suite_in_statement` is `false` for the suite containing `foo10` and `foo12`
+        /// and `true` for the suite containing `bar`.
+        /// ```python
+        /// if sys.version_info >= (3, 10):
+        ///     def foo10():
+        ///         return "new"
+        /// elif sys.version_info >= (3, 12):
+        ///     def foo12():
+        ///         return "new"
+        /// else:
+        ///     def bar():
+        ///         return "old"
+        /// ```
+        ///
+        /// When this value is true, we don't insert trailing empty lines since the containing suite
+        /// will do that.
+        last_suite_in_statement: bool,
+    },
 }
 
-#[derive(Debug)]
-pub struct FormatSuite {
-    kind: SuiteKind,
-}
-
-impl Default for FormatSuite {
+impl Default for SuiteKind {
     fn default() -> Self {
-        FormatSuite {
-            kind: SuiteKind::Other,
+        Self::Other {
+            // For stability, we can't insert an empty line if we don't know if the outer suite
+            // also does.
+            last_suite_in_statement: true,
         }
     }
 }
 
+impl SuiteKind {
+    /// See [`SuiteKind::Other`].
+    pub fn other(last_suite_in_statement: bool) -> Self {
+        Self::Other {
+            last_suite_in_statement,
+        }
+    }
+
+    pub fn last_suite_in_statement(self) -> bool {
+        match self {
+            Self::Other {
+                last_suite_in_statement,
+            } => last_suite_in_statement,
+            _ => true,
+        }
+    }
+}
+
+#[derive(Debug, Default)]
+pub struct FormatSuite {
+    kind: SuiteKind,
+}
+
 impl FormatRule<Suite, PyFormatContext<'_>> for FormatSuite {
     fn fmt(&self, statements: &Suite, f: &mut PyFormatter) -> FormatResult<()> {
         let mut iter = statements.iter();
@@ -64,7 +106,7 @@ impl FormatRule<Suite, PyFormatContext<'_>> for FormatSuite {
                         TopLevelStatementPosition::Other
                     }),
             ),
-            SuiteKind::Function | SuiteKind::Class | SuiteKind::Other => {
+            SuiteKind::Function | SuiteKind::Class | SuiteKind::Other { .. } => {
                 NodeLevel::CompoundStatement
             }
         };
@@ -78,7 +120,7 @@ impl FormatRule<Suite, PyFormatContext<'_>> for FormatSuite {
 
         // Format the first statement in the body, which often has special formatting rules.
         let first = match self.kind {
-            SuiteKind::Other => {
+            SuiteKind::Other { .. } => {
                 if is_class_or_function_definition(first)
                     && !comments.has_leading(first)
                     && !source_type.is_stub()
@@ -99,7 +141,7 @@ impl FormatRule<Suite, PyFormatContext<'_>> for FormatSuite {
 
             SuiteKind::Function | SuiteKind::Class | SuiteKind::TopLevel => {
                 if let Some(docstring) =
-                    DocstringStmt::try_from_statement(first, self.kind, source_type)
+                    DocstringStmt::try_from_statement(first, self.kind, f.context())
                 {
                     SuiteChildStatement::Docstring(docstring)
                 } else {
@@ -129,9 +171,24 @@ impl FormatRule<Suite, PyFormatContext<'_>> for FormatSuite {
                 false,
             )
         } else {
+            // Allow an empty line after a function header in preview, if the function has no
+            // docstring and no initial comment.
+            let allow_newline_after_block_open =
+                is_allow_newline_after_block_open_enabled(f.context())
+                    && matches!(self.kind, SuiteKind::Function)
+                    && matches!(first, SuiteChildStatement::Other(_));
+
+            let start = comments
+                .leading(first)
+                .first()
+                .map_or_else(|| first.start(), Ranged::start);
+
+            if allow_newline_after_block_open && lines_before(start, f.context().source()) > 1 {
+                empty_line().fmt(f)?;
+            }
+
             first.fmt(f)?;
 
-            #[allow(clippy::if_same_then_else)]
             let empty_line_after_docstring = if matches!(first, SuiteChildStatement::Docstring(_))
                 && self.kind == SuiteKind::Class
             {
@@ -140,7 +197,7 @@ impl FormatRule<Suite, PyFormatContext<'_>> for FormatSuite {
                 // Insert a newline after a module level docstring, but treat
                 // it as a docstring otherwise. See: https://github.com/psf/black/pull/3932.
                 self.kind == SuiteKind::TopLevel
-                    && DocstringStmt::try_from_statement(first.statement(), self.kind, source_type)
+                    && DocstringStmt::try_from_statement(first.statement(), self.kind, f.context())
                         .is_some()
             };
 
@@ -161,55 +218,7 @@ impl FormatRule<Suite, PyFormatContext<'_>> for FormatSuite {
                 // Here we insert empty lines even if the preceding has a trailing own line comment
                 true
             } else {
-                // Find nested class or function definitions that need an empty line after them.
-                //
-                // ```python
-                // def f():
-                //     if True:
-                //
-                //         def double(s):
-                //             return s + s
-                //
-                //     print("below function")
-                // ```
-                std::iter::successors(
-                    Some(AnyNodeRef::from(preceding)),
-                    AnyNodeRef::last_child_in_body,
-                )
-                .take_while(|last_child|
-                    // If there is a comment between preceding and following the empty lines were
-                    // inserted before the comment by preceding and there are no extra empty lines
-                    // after the comment.
-                    // ```python
-                    // class Test:
-                    //     def a(self):
-                    //         pass
-                    //         # trailing comment
-                    //
-                    //
-                    // # two lines before, one line after
-                    //
-                    // c = 30
-                    // ````
-                    // This also includes nested class/function definitions, so we stop recursing
-                    // once we see a node with a trailing own line comment:
-                    // ```python
-                    // def f():
-                    //     if True:
-                    //
-                    //         def double(s):
-                    //             return s + s
-                    //
-                    //         # nested trailing own line comment
-                    //     print("below function with trailing own line comment")
-                    // ```
-                    !comments.has_trailing_own_line(*last_child))
-                .any(|last_child| {
-                    matches!(
-                        last_child,
-                        AnyNodeRef::StmtFunctionDef(_) | AnyNodeRef::StmtClassDef(_)
-                    )
-                })
+                trailing_function_or_class_def(Some(preceding), &comments).is_some()
             };
 
             // Add empty lines before and after a function or class definition. If the preceding
@@ -227,7 +236,7 @@ impl FormatRule<Suite, PyFormatContext<'_>> for FormatSuite {
                     )?;
                 } else {
                     // Preserve empty lines after a stub implementation but don't insert a new one if there isn't any present in the source.
-                    // This is useful when having multiple function overloads that should be grouped to getter by omitting new lines between them.
+                    // This is useful when having multiple function overloads that should be grouped together by omitting new lines between them.
                     let is_preceding_stub_function_without_empty_line = following
                         .is_function_def_stmt()
                         && preceding
@@ -248,7 +257,7 @@ impl FormatRule<Suite, PyFormatContext<'_>> for FormatSuite {
                             SuiteKind::TopLevel => {
                                 write!(f, [empty_line(), empty_line()])?;
                             }
-                            SuiteKind::Function | SuiteKind::Class | SuiteKind::Other => {
+                            SuiteKind::Function | SuiteKind::Class | SuiteKind::Other { .. } => {
                                 empty_line().fmt(f)?;
                             }
                         }
@@ -280,7 +289,7 @@ impl FormatRule<Suite, PyFormatContext<'_>> for FormatSuite {
                             },
                         }
                     }
-                    SuiteKind::Function | SuiteKind::Class | SuiteKind::Other => {
+                    SuiteKind::Function | SuiteKind::Class | SuiteKind::Other { .. } => {
                         empty_line().fmt(f)?;
                     }
                 }
@@ -319,7 +328,7 @@ impl FormatRule<Suite, PyFormatContext<'_>> for FormatSuite {
                                 write!(f, [empty_line(), empty_line()])?;
                             }
                         },
-                        SuiteKind::Function | SuiteKind::Class | SuiteKind::Other => {
+                        SuiteKind::Function | SuiteKind::Class | SuiteKind::Other { .. } => {
                             empty_line().fmt(f)?;
                         }
                     },
@@ -413,10 +422,129 @@ impl FormatRule<Suite, PyFormatContext<'_>> for FormatSuite {
             empty_line_after_docstring = false;
         }
 
+        self.between_alternative_blocks_empty_line(statements, &comments, f)?;
+
         Ok(())
     }
 }
 
+impl FormatSuite {
+    /// Add an empty line between a function or class and an alternative body.
+    ///
+    /// We only insert an empty if we're between suites in a multi-suite statement. In the
+    /// if-else-statement below, we insert an empty line after the `foo` in the if-block, but none
+    /// after the else-block `foo`, since in the latter case the enclosing suite already adds
+    /// empty lines.
+    ///
+    /// ```python
+    /// if sys.version_info >= (3, 10):
+    ///     def foo():
+    ///         return "new"
+    /// else:
+    ///     def foo():
+    ///         return "old"
+    /// class Bar:
+    ///     pass
+    /// ```
+    fn between_alternative_blocks_empty_line(
+        &self,
+        statements: &Suite,
+        comments: &Comments,
+        f: &mut PyFormatter,
+    ) -> FormatResult<()> {
+        if self.kind.last_suite_in_statement() {
+            // If we're at the end of the current statement, the outer suite will insert one or
+            // two empty lines already.
+            return Ok(());
+        }
+
+        let Some(last_def_or_class) = trailing_function_or_class_def(statements.last(), comments)
+        else {
+            // An empty line is only inserted for function and class definitions.
+            return Ok(());
+        };
+
+        // Skip the last trailing own line comment of the suite, if any, otherwise we count
+        // the lines wrongly by stopping at that comment.
+        let node_with_last_trailing_comment = std::iter::successors(
+            statements.last().map(AnyNodeRef::from),
+            AnyNodeRef::last_child_in_body,
+        )
+        .find(|last_child| comments.has_trailing_own_line(*last_child));
+
+        let end_of_def_or_class = node_with_last_trailing_comment
+            .and_then(|child| comments.trailing(child).last().map(Ranged::end))
+            .unwrap_or(last_def_or_class.end());
+        let existing_newlines =
+            lines_after_ignoring_end_of_line_trivia(end_of_def_or_class, f.context().source());
+        if existing_newlines < 2 {
+            if f.context().is_preview() {
+                empty_line().fmt(f)?;
+            } else {
+                if last_def_or_class.is_stmt_class_def() && f.options().source_type().is_stub() {
+                    empty_line().fmt(f)?;
+                }
+            }
+        }
+        Ok(())
+    }
+}
+
+/// Find nested class or function definitions that need an empty line after them.
+///
+/// ```python
+/// def f():
+///     if True:
+///
+///         def double(s):
+///             return s + s
+///
+///     print("below function")
+/// ```
+fn trailing_function_or_class_def<'a>(
+    preceding: Option<&'a Stmt>,
+    comments: &Comments,
+) -> Option<AnyNodeRef<'a>> {
+    std::iter::successors(
+        preceding.map(AnyNodeRef::from),
+        AnyNodeRef::last_child_in_body,
+    )
+    .take_while(|last_child|
+        // If there is a comment between preceding and following the empty lines were
+        // inserted before the comment by preceding and there are no extra empty lines
+        // after the comment.
+        // ```python
+        // class Test:
+        //     def a(self):
+        //         pass
+        //         # trailing comment
+        //
+        //
+        // # two lines before, one line after
+        //
+        // c = 30
+        // ````
+        // This also includes nested class/function definitions, so we stop recursing
+        // once we see a node with a trailing own line comment:
+        // ```python
+        // def f():
+        //     if True:
+        //
+        //         def double(s):
+        //             return s + s
+        //
+        //         # nested trailing own line comment
+        //     print("below function with trailing own line comment")
+        // ```
+        !comments.has_trailing_own_line(*last_child))
+    .find(|last_child| {
+        matches!(
+            last_child,
+            AnyNodeRef::StmtFunctionDef(_) | AnyNodeRef::StmtClassDef(_)
+        )
+    })
+}
+
 /// Stub files have bespoke rules for empty lines.
 ///
 /// These rules are ported from black (preview mode at time of writing) using the stubs test case:
@@ -447,7 +575,7 @@ fn stub_file_empty_lines(
                 hard_line_break().fmt(f)
             }
         }
-        SuiteKind::Class | SuiteKind::Other | SuiteKind::Function => {
+        SuiteKind::Class | SuiteKind::Other { .. } | SuiteKind::Function => {
             if (empty_line_condition
                 && lines_after_ignoring_end_of_line_trivia(preceding.end(), source) > 1)
                 || require_empty_line
@@ -477,26 +605,14 @@ pub(crate) fn should_insert_blank_line_after_class_in_stub_file(
         return false;
     }
 
+    let Some(following) = following else {
+        // We handle newlines at the end of a suite in `between_alternative_blocks_empty_line`.
+        return false;
+    };
+
     let comments = context.comments();
     match preceding.as_stmt_class_def() {
         Some(class) if contains_only_an_ellipsis(&class.body, comments) => {
-            let Some(following) = following else {
-                // The formatter is at the start of an alternate branch such as
-                // an `else` block.
-                //
-                // ```python
-                // if foo:
-                //     class Nested:
-                //         pass
-                // else:
-                //     pass
-                // ```
-                //
-                // In the above code, the preceding node is the `Nested` class
-                // which has no following node.
-                return true;
-            };
-
             // If the preceding class has decorators, then we need to add an empty
             // line even if it only contains ellipsis.
             //
@@ -603,10 +719,15 @@ fn stub_suite_can_omit_empty_line(preceding: &Stmt, following: &Stmt, f: &PyForm
     //
     // class LockType2: ...
     // ```
-    let class_decorator_instead_of_empty_line = preceding.is_function_def_stmt()
-        && following
-            .as_class_def_stmt()
-            .is_some_and(|class| !class.decorator_list.is_empty());
+    //
+    // However, this behavior is incorrect and should not be replicated in preview mode.
+    // See: https://github.com/astral-sh/ruff/issues/18865
+    let class_decorator_instead_of_empty_line =
+        !is_blank_line_before_decorated_class_in_stub_enabled(f.context())
+            && preceding.is_function_def_stmt()
+            && following
+                .as_class_def_stmt()
+                .is_some_and(|class| !class.decorator_list.is_empty());
 
     // A function definition following a stub function definition
     // ```python
@@ -625,17 +746,21 @@ fn stub_suite_can_omit_empty_line(preceding: &Stmt, following: &Stmt, f: &PyForm
 
 /// Returns `true` if a function or class body contains only an ellipsis with no comments.
 pub(crate) fn contains_only_an_ellipsis(body: &[Stmt], comments: &Comments) -> bool {
-    match body {
-        [Stmt::Expr(ast::StmtExpr { value, .. })] => {
-            let [node] = body else {
-                return false;
-            };
-            value.is_ellipsis_literal_expr()
-                && !comments.has_leading(node)
-                && !comments.has_trailing_own_line(node)
-        }
-        _ => false,
+    as_only_an_ellipsis(body, comments).is_some()
+}
+
+/// Returns `Some(Stmt::Ellipsis)` if a function or class body contains only an ellipsis with no
+/// comments.
+pub(crate) fn as_only_an_ellipsis<'a>(body: &'a [Stmt], comments: &Comments) -> Option<&'a Stmt> {
+    if let [node @ Stmt::Expr(ast::StmtExpr { value, .. })] = body
+        && value.is_ellipsis_literal_expr()
+        && !comments.has_leading(node)
+        && !comments.has_trailing_own_line(node)
+    {
+        return Some(node);
     }
+
+    None
 }
 
 /// Returns `true` if a [`Stmt`] is a class or function definition.
@@ -687,37 +812,23 @@ impl<'a> DocstringStmt<'a> {
     fn try_from_statement(
         stmt: &'a Stmt,
         suite_kind: SuiteKind,
-        source_type: PySourceType,
+        context: &PyFormatContext,
     ) -> Option<DocstringStmt<'a>> {
         // Notebooks don't have a concept of modules, therefore, don't recognise the first string as the module docstring.
-        if source_type.is_ipynb() && suite_kind == SuiteKind::TopLevel {
+        if context.options().source_type().is_ipynb() && suite_kind == SuiteKind::TopLevel {
             return None;
         }
 
-        let Stmt::Expr(ast::StmtExpr { value, .. }) = stmt else {
-            return None;
-        };
-
-        match value.as_ref() {
-            Expr::StringLiteral(ast::ExprStringLiteral { value, .. })
-                if !value.is_implicit_concatenated() =>
-            {
-                Some(DocstringStmt {
-                    docstring: stmt,
-                    suite_kind,
-                })
-            }
-            _ => None,
-        }
+        Self::is_docstring_statement(stmt.as_expr_stmt()?, context).then_some(DocstringStmt {
+            docstring: stmt,
+            suite_kind,
+        })
     }
 
-    pub(crate) fn is_docstring_statement(stmt: &StmtExpr, source_type: PySourceType) -> bool {
-        if source_type.is_ipynb() {
-            return false;
-        }
-
+    pub(crate) fn is_docstring_statement(stmt: &StmtExpr, context: &PyFormatContext) -> bool {
         if let Expr::StringLiteral(ast::ExprStringLiteral { value, .. }) = stmt.value.as_ref() {
             !value.is_implicit_concatenated()
+                || !value.iter().any(|literal| context.comments().has(literal))
         } else {
             false
         }
@@ -752,7 +863,7 @@ impl Format<PyFormatContext<'_>> for DocstringStmt<'_> {
                         .then_some(source_position(self.docstring.start())),
                     string_literal
                         .format()
-                        .with_options(ExprStringLiteralKind::Docstring),
+                        .with_options(StringLiteralKind::Docstring),
                     f.options()
                         .source_map_generation()
                         .is_enabled()
@@ -833,10 +944,10 @@ mod tests {
     use ruff_python_parser::parse_module;
     use ruff_python_trivia::CommentRanges;
 
+    use crate::PyFormatOptions;
     use crate::comments::Comments;
     use crate::prelude::*;
     use crate::statement::suite::SuiteKind;
-    use crate::PyFormatOptions;
 
     fn format_suite(level: SuiteKind) -> String {
         let source = r"
@@ -916,7 +1027,9 @@ def trailing_func():
 
     #[test]
     fn nested_level() {
-        let formatted = format_suite(SuiteKind::Other);
+        let formatted = format_suite(SuiteKind::Other {
+            last_suite_in_statement: true,
+        });
 
         assert_eq!(
             formatted,
